import binascii
from typing import TYPE_CHECKING

import requests
from constance import config
from django import forms
from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.forms import MultiWidget
from django.utils.translation import gettext as _
from requests.auth import HTTPBasicAuth
from requests.exceptions import MissingSchema, ReadTimeout

from aurora.core.fields.mixins import MultiValueWidgetMixin
from aurora.core.fields.widgets import SmartTextWidget
from aurora.core.version_media import VersionMedia

from .mixins import ConfigurableSmartField

if TYPE_CHECKING:
    from aurora.core.models import FlexFormField

FALSE = "false"

STERLING_BANK = "000001"
KEYSTONE_BANK = "000002"
FCMB = "000003"
UNITED_BANK_FOR_AFRICA = "000004"
JAIZ_BANK = "000006"
FIDELITY_BANK = "000007"
POLARIS_BANK = "000008"
CITI_BANK = "000009"
ECOBANK_BANK = "000010"
UNITY_BANK = "000011"
STANBICIBTC_BANK = "000012"
GTBANK_PLC = "000013"
ACCESS_BANK = "000014"
ZENITH_BANK = "000015"
FIRST_BANK_OF_NIGERIA = "000016"
WEMA_BANK = "000017"
UNION_BANK = "000018"
ENTERPRISE_BANK = "000019"
HERITAGE = "000020"
STANDARDCHARTERED = "000021"
FORTIS_MICROFINANCE_BANK = "070002"
COVENANT = "070006"
ASOSAVINGS = "090001"
JUBILEELIFE = "090003"
TRUSTBOND_MORTGAGE_BANK_PLC = "090005"
FETS = "100001"
PAGATECH = "100002"
PARKWAY_READYCASH = "100003"
OPAY_DIGITAL_SERVICES_LIMITED = "100004"
ETRANZACT = "100006"
STANBIC_IBTC_EASE_WALLET = "100007"
ECOMOBILE = "100008"
GTMOBILE = "100009"
TEASYMOBILE = "100010"
MKUDI = "100011"
VTNETWORKS = "100012"
ACCESS_MONEY = "100013"
FIRSTMONIE_WALLET = "100014"
KEGOW_CHAMSMOBILE = "100036"
FORTISMOBILE = "100016"
HEDONMARK = "100017"
ZENITHMOBILE = "100018"
FIDELITY_MOBILE = "100019"
MONEYBOX = "100020"
EARTHOLEUM = "100021"
FSDH = "400001"
NIP_VIRTUAL_BANK = "999999"
NPF_MICROFINANCE_BANK = "070001"
SAFETRUST_MORTAGE = "090006"
CORONATION_MERCHANT_BANK = "060001"
TAGPAY_VISUAL_ICT = "100023"
GOMONEY = "100022"
SUNTRUST_BANK = "000022"
PAYATTITUDE_ONLINE = "110001"
OMOLUABI = "070077"
OMOLUABI = "070007"
PAGE_MFBANK = "070008"
FIRST_MORTGAGE_LIMITED = "090107"
NEW_PRUDENTIAL_BANK = "090108"
IMPERIAL_HOMES_MORTGAGE_BANK = "100024"
CARBON_MFB = "100026"
INTELLIFIN = "100027"
PROVIDUS_BANK = "000023"
VFD_MICROFINANCE_BANK = "090110"
EKONDO_MICROFINANCE_BANK = "090097"
BOCTRUST_MICROFINANCE_BANK_LIMITED = "090117"
RAND_MERCHANT_BANK = "000024"
TCF_MICROFINANCE_BANK_LIMITED = "090115"
KONGAPAY = "100025"
AG_MORTGAGE_BANK_PLC = "100028"
IBILE_MICROFINANCE_BANK_LIMITED = "090118"
PARRALEX_MICROFINANCE_BANK = "90004"
FBNQUEST_MERCHANT_BANK = "060002"
GATEWAY_MORTGAGE_BANK = "070009"
ABBEY_MORTGAGE_BANK = "070010"
FINATRUST_MICROFINANCE_BANK = "090111"
SEED_CAPITAL_MICROFINANCE_BANK = "090112"
EMPIRE_MFB = "090114"
AMML_MFB = "090116"
OHAFIA_MFB = "090119"
WETLAND_MFB = "090120"
HASAL_MFB = "090121"
GOWANS_MFB = "090122"
TRUSTBANC_J6_MICROFINANCE_BANK_LIMITED = "090123"
XSLNCE_MICROFINANCE_BANK = "090124"
REGENT_MFB = "090125"
FIDFUND_MFB = "090126"
BC_KASH_MFB = "090127"
NDIORAH_MFB = "090128"
MONEYTRUST_MFB = "090129"
CONSUMER__MFB = "090130"
INNOVECTIVES_KESH = "100029"
REFUGE_MORTGAGE_BANK = "070011"
LBIC_MORTGAGE_BANK = "070012"
PLATINUM_MORTGAGE_BANK = "070013"
FIRST_GENERATION_MORTGAGE_BANK = "070014"
BRENT_MORTGAGE_BANK = "070015"
INFINITY_TRUST__MORTGAGE_BANK = "070016"
ALLWORKERS_MFB = "090131"
RICHWAY_MFB = "090132"
AL_BARKAH_MFB = "090133"
ACCION_MFB = "090134"
PERSONAL_TRUST_MICROFINANCE_BANK = "090135"
BAOBAB_MICROFINANCE_BANK = "090136"
PECAN_TRUST_MICROFINANCE_BANK = "090137"
ROYAL_EXCHANGE_MICROFINANCE_BANK = "090138"
VISA_MICROFINANCE_BANK = "090139"
SAGAMU_MICROFINANCE_BANK = "090140"
CHIKUM_MICROFINANCE_BANK = "090141"
YES_MFB = "090142"
APEKS_MICROFINANCE_BANK = "090143"
MONEYFIELD_MICROFINANCE_BANK = "090144"
FULL_RANGE_MFB = "090145"
TRIDENT_MICROFINANCE_BANK = "090146"
HACKMAN_MICROFINANCE_BANK = "090147"
BOWEN_MFB = "090148"
IRL_MICROFINANCE_BANK = "090149"
VIRTUE_MFB = "090150"
MUTUAL_TRUST_MICROFINANCE_BANK = "090151"
NARGATA_MFB = "090152"
FFS_MICROFINANCE_BANK = "090153"
ADVANS_LAFAYETTE__MFB = "090155"
E_BARCS_MFB = "090156"
INFINITY_MFB = "090157"
FUTO_MFB = "090158"
ADDOSSER_MFBB = "090160"
FCMB_MOBILE = "100031"
FLUTTERWAVE_TECHNOLOGY_SOLUTIONS_LIMITED = "110002"
NOVA_MERCHANT_BANK = "060003"
FEDERAL_UNIVERSITY_DUTSE__MICROFINANCE_BANK = "090318"
KADPOLY_MICROFINANCE_BANK = "090320"
MAYFAIR_MFB = "090321"
REPHIDIM_MICROFINANCE_BANK = "090322"
IKENNE_MFB = "090324"
TRUST_MFB = "090327"
UNAAB_MFB = "090331"
EVERGREEN_MICROFINANCE_BANK = "090332"
BIPC_MICROFINANCE_BANK = "090336"
CASHCONNECT_MICROFINANCE_BANK = "090360"
NUTURE_MFB = "090364"
ILASAN_MICROFINANCE_BANK = "090370"
AGOSASA_MICROFINANCE_BANK = "090371"
LEGEND_MICROFINANCE_BANK = "090372"
TF_MICROFINANCE_BANK = "090373"
COASTLINE_MICROFINANCE_BANK = "090374"
NEPTUNE_MFB = "090329"
MOLUSI_MFB = "090362"
MANNY_MFB = "090383"
CORESTEP_MICROFINANCE_BANK = "090365"
SEEDVEST_MICROFINANCE_BANK = "090369"
APPLE__MICROFINANCE_BANK = "090376"
ISALEOYO_MICROFINANCE_BANK = "090377"
NEW_GOLDEN_PASTURES_MICROFINANCE_BANK = "090378"
GTI__MICROFINANCE_BANK = "090385"
EK_RELIABLE_MICROFINANCE_BANK = "090389"
DAVODANI__MICROFINANCE_BANK = "090391"
VENTURE_GARDEN_NIGERIA_LIMITED = "110009"
ARCA_PAYMENTS_COMPANY_LIMITED = "110011"
MOZFIN_MICROFINANCE_BANK = "090392"
PAYMENT_SERVICE_BANK = "120001"
INTERSWITCH_LIMITED = "110003"
PAYSTACK_PAYMENTS = "110006"
TEAM_APT = "110007"
ITEX_INTEGRATED_SERVICES_LIMITED = "090211"
BORGU_MICROFINANCE_BANK = "090395"
HOPE_PAYMENT_SERVICE_BANK = "120002"
MONIEPOINT_MFB = "090405"
FEDERAL_POLYTECHNIC_NEKEDE_MICROFINANCE_BANK = "090398"
MARITIME_MICROFINANCE_BANK = "090410"
GMB_MICROFINANCE_BANK = "090408"
CHANELLE_MFB = "090397"
GIGINYA_MFB = "090411"
CHIBUEZE__MFB = "090416"
PREEMINENT_MICROFINANCE_BANK = "090412"
COOP_MORTGAGE_BANK = "070021"
LANDGOLD_MICROFINANCE_BANK = "090422"
UNN_MFB = "090251"
ABUCOOP_MICROFINANCE_BANK = "090424"
FIRMUS_MFB = "090366"
BENYSTA_MICROFINANCE_BANK = "090413"
PEACE_MICROFINANCE_BANK = "090402"
QR_PAYMENTS = "110013"
HIGHLAND_MICROFINANCE_BANK = "090418"
CENTRAL_BANK_OF_NIGERIA = "000028"
MAUTECH_MICROFINANCE_BANK = "090423"
TANGERINE_MONEY = "090426"
ISHIE_MICROFINANCE_BANK = "090428"
ILORA_MICROFINANCE_BANK = "090430"
BLUEWHALES_MICROFINANCE_BANK = "090431"
IZON_MICROFINANCE_BANK = "090421"
BANEX_MICROFINANCE_BANK = "090425"
LTSHEGO_MICROFINANCE_BANK = "090420"
SUNBEAM_MICROFINANCE_BANK = "090302"
WINVIEW_BANK = "090419"
CALABAR_MICROFINANCE_BANK = "090415"
IMOWO_MICROFINANCE_BANK = "090417"
FCMB_BETA = "090409"
BUSINESS_SUPPORTMICROFINANCE_BANK = "090406"
SHEPHERD_TRUST_MICROFINANCE_BANK = "090401"
DOJE_MICROFINANCE_BANK_LIMITED = "090404"
PETRA_MICROFINANCE_BANK_PLC = "090165"
HAGGAI_MORTGAGE_BANK_LIMITED = "070017"
MALACHY_MICROFINANCE_BANK = "090174"
MAINSTREET_MICROFINANCE_BANK = "090171"
RUBIES_MICROFINANCE_BANK = "090175"
FAST_MICROFINANCE_BANK = "090179"
AMJU_UNIQUE_MICROFINANCE_BANK_ = "090180"
ALPHA_KAPITAL_MFB_LTD = "090169"
RENMONEY_MICROFINANCE_BANK = "090198"
BAINES_CREDIT_MICROFINANCE_BANK = "090188"
NIGERIAN_NAVY_MICROFINANCE_BANK = "090263"
KUDA_MICROFINANCE_BANK = "090267"
NIRSAL_MICROFINANCE_BANK = "090194"
TRUSTFUND_MICROFINANCE_BANK = "090276"
RELIANCE_MFB = "090173"
NEW_DAWN_MICROFINANCE_BANK = "090205"
ESAN_MICROFINANCE_BANK = "090189"
OLABISI_ONABANJO_UNIVERSITY_MICROFINANCE_BANK = "090272"
LINE_CARD_MANAGEMENT_LIMITED = "110005"
TITAN_TRUST_BANK = "000025"
FIRST_APPLE_LIMITED = "110004"
GLORY_MICROFINANCE_BANK = "090278"
GLOBUS_BANK = "000027"
TAJ_BANK = "000026"
AL_HAYAT_MICROFINANCE_BANK = "090277"
AB_MICROFINANCE_BANK = "090270"
ABOVE_ONLY_MICROFINANCE_BANK = "090260"
ABU_MICROFINANCE_BANK = "090197"
ADEYEMI_COLLEGE_STAFF_MICROFINANCE_BANK = "090268"
ALEKUN_MICROFINANCE_BANK = "090259"
AUCHI_MICROFINANCE_BANK = "090264"
EMERALDS_MFB = "090273"
GREENVILLE_MICROFINANCE_BANK = "090269"
GROOMING_MICROFINANCE_BANK = "090195"
IMO_MICROFINANCE_BANK = "090258"
UNICAL_MFB = "090193"
LAVENDER_MICROFINANCE_BANK = "090271"
LOVONUS_MICROFINANCE_BANK = "090265"
KCMB_MFB = "090191"
PENNYWISE_MICROFINANCE_BANK = "090196"
QUICKFUND_MICROFINANCE_BANK = "090261"
STELLAS_MICROFINANCE_BANK = "090262"
UNIBEN_MICROFINANCE_BANK = "090266"
POLYUWANNA_MFB = "090296"
ALERT_MFB = "090297"
CEMCS_MFB = "090154"
CREDIT_AFRIQUE_MFB = "090159"
OKPOGA_MFB = "090161"
STANFORD_MFB = "090162"
FIRST_MULTIPLE_MFB = "090163"
MERIDIAN_MFB = "090275"
IKIRE_MFB = "090279"
MEGAPRAISE_MICROFINANCE_BANK = "090280"
MINT_FINEX_MICROFINANCE = "090281"
ARISE_MFB = "090282"
THRIVE_MFB = "090283"
FIRST_OPTION_MFB = "090285"
SAFE_HAVEN_MFB = "090286"
PILLAR_MFB = "090289"
FCT_MFB = "090290"
HALA_MFB = "090291"
EAGLE_FLIGHT_MFB = "090294"
OMIYE_MFB = "090295"
ESO_E_MICROFINANCE_BANK = "090166"
DAYLIGHT_MICROFINANCE_BANK = "090167"
GASHUA_MICROFINANCE_BANK = "090168"
RAHAMA_MFB = "090170"
ASTRAPOLIS_MFB = "090172"
BOSAK_MFB = "090176"
PRESTIGE_MICROFINANCE_BANK = "090274"
GREENBANK_MFB = "090178"
MUTUAL_BENEFIT_MFB = "090190"
ACCESS_YELLO_ = "100052"
PALMPAY = "100033"
AFEKHAFE_MICROFINANCE_BANK = "090292"
SPARKLE = "090325"
PATRICKGOLD_MICROFINANCE_BANK = "090317"
MAINLAND_MICROFINANCE_BANK = "090323"
BALOGUN_GAMBARI_MICROFINANCE_BANK = "090326"
KADICK_INTEGRATION_LIMITED = "110008"
LAPO_MFB = "090177"
EYOWO_MFB = "090328"
BRETHREN_MICROFINANCE_BANK = "090293"
OCHE_MICROFINANCE_BANK = "090333"
ACCELEREX_NETWORK = "090202"
MAYFRESH_MORTGAGE_BANK = "070019"
XPRESS_PAYMENTS = "090201"
FEDERALPOLY_NASARAWAMFB = "090298"
KONTAGORA_MFB = "090299"
PURPLEMONEY_MFB = "090303"
EVANGEL_MFB = "090304"
SULSAP_MFB = "090305"
BRIGHTWAY_MFB = "090308"
EDFIN_MFB = "090310"
U_AND_C_MICROFINANCE_BANK = "090315"
BAYERO_MICROFINANCE_BANK = "090316"
AMAC_MICROFINANCE_BANK = "090394"
FINCA_MICROFINANCE_BANK = "090400"
BRIDGEWAY_MICROFINANCE_BANK = "090393"
OSCOTECH_MFB = "090396"
KREDI_MONEY_MICROFINANCE_BANK = "090380"
YOBE_MFB = "090252"
NWANNEGADI_MFB = "090399"
GIREI_MFB = "090186"
INTERLAND_MFB = "090386"
HEADWAY_MFB = "090363"
ASSETS_MATRIX_MFB = "090287"
MIDLAND_MFB = "090192"
MICROVIS_MICROFINANCE_BANK = "090113"
M36 = "100035"
ZENITH_EAZY_WALLET = "100034"
CYBERSPACE_LIMITED = "110014"
CELLULANT_PSSP = "110012"
LINKS_MICROFINANCE_BANK = "090435"
EBSU_MFB = "090427"
STB_MORTGAGE_BANK = "070022"
MEMPHIS_MICROFINANCE_BANK = "090432"
FUTMINNA_MICROFINANCE_BANK_ = "090438"
RIGO_MICROFINANCE_BANK = "090433"
UNILORIN_MICROFINANCE_BANK = "090341"
GREENWICH_MERCHANT_BANK = "060004"
LOTUS_BANK = "000029"
FAME_MICROFINANCE_BANK = "090330"
INSIGHT_MICROFINANCE_BANK = "090434"
SPECTRUM_MICROFINANCE_BANK = "090436"
BORSTAL_MICROFINANCE_BANK = "090454"
OSPOLY_MICROFINANCE_BANK = "090456"
CAPSTONE_MICROFINANCE_BANK = "090445"
PARALLEX_BANK = "000030"
KWASU_MICROFINANCE_BANK = "090450"
NICE_MICROFINANCE_BANK = "090459"
IBETO_MICROFINANCE_BANK = "090439"
BALOGUN_FULANI_MICROFINANCE_BANK = "090181"
BONGHE_MICROFINANCE_BANK = "090319"
GRANT_MICROFINANCE_BANK = "090335"
STOCKCORP__MICROFINANCE_BANK = "090340"
ILLORIN_MICROFINANCE_BANK = "090350"
PARKWAY_MF_BANK = "090390"
CRUTECH__MICROFINANCE_BANK = "090414"
OAKLAND_MICROFINANCE_BANK = "090437"
CHERISH_MICROFINANCE_BANK = "090440"
GIWA_MICROFINANCE_BANK = "090441"
RIMA_MICROFINANCE_BANK = "090443"
SUPPORT_MF_BANK = "090446"
MOYOFADE_MF_BANK = "090448"
REX_MICROFINANCE_BANK = "090449"
ATBU__MICROFINANCE_BANK = "090451"
UNILAG__MICROFINANCE_BANK = "090452"
UZONDU_MF_BANK = "090453"
COALCAMP_MICROFINANCE_BANK = "090254"
UNIUYO_MICROFINANCE_BANK = "090338"
CROSSRIVER__MICROFINANCE_BANK = "090429"
ORITA_BASORUN_MICROFINANCE_BANK = "090460"
UNIIBADAN_MICROFINANCE_BANK = "090461"
MONARCH_MICROFINANCE_BANK = "090462"
REHOBOTH_MICROFINANCE_BANK = "090463"
UNIMAID_MICROFINANCE_BANK = "090464"
MAINTRUST_MFB = "090465"
GOOD_NEIGHBOURS_MICROFINANCE_BANK = "090467"
OLOFIN_OWENA_MICROFINANCE_BANK = "090468"
CARETAKER_MICROFINANCE_BANK = "090472"
ASSETS_MICROFINANCE_BANK = "090473"
SAFEGATE_MICROFINANCE_BANK = "090485"
INTERSWITCH_FINANCIAL_INCLUSION_SERVICES_IFIS = "110010"
OLUCHUKWU_MICROFINANCE_BANK = "090471"
VAS2NETS_LIMITED = "110015"
CROWDFORCE = "110017"
CHANGAN_RTS_MICROFINANCE_BANK = "090470"
LIGHT_MICROFINANCE_BANK = "090477"
AVUENEGBE_MFB = "090478"
FEDETH_MFB = "090482"
PRISCO_MFB = "090481"
CINTRUST_MFB = "090480"
ADA_MFB = "090483"
ANIOCHA_MFB = "090469"
ALVANA_MICROFINANCE_BANK = "090489"
GARKI_MFB = "090484"
FIRST_HERITAGE_MFB = "090479"
FOTRESS_MICROFINANCE_BANK = "090486"
ANCHORAGE_MFB = "90476"
MICROSYSTEMS_INVESTMENT_AND_DEVELOPMENT_LIMITED = "110018"
NIBSSUSSD_PAYMENTS = "110019"
PRISTINE_DIVITIS_MFB = "090499"
GOODNEWS_MICROFINANCE_BANK = "090495"
CHUKWUNENYE__MICROFINANCE_BANK = "090490"
ORAUKWU_MICROFINANCE_BANK = "090492"
KINGDOM_COLLEGE_MICROFINANCE_BANK = "090487"
JESSEFIELD_MICROFINANCE = "090352"
DELTA_TRUST_MORTGAGE_BANK = "070023"
IPERUA_MICROFINANCE_BANK = "090493"
SHALOM_MICROFINACE_BANK = "090502"
NSUK_MICROFINANCE_BANK = "090491"
BOJIA_BOJIA_MICROFINANCE_BANK = "090494"
ZIKORA_MICROFINANCE_BANK = "090504"
TAJ_PINSPAY = "080002"
PROJECTS_MICROFINANCE_BANK = "090503"
NOWNOW_MODIFICATION = "100032"
CITIZEN_TRUST_A_MFB = "090343"
FIRST_ROYAL_MICROFINANCE_BANK = "090164"
PALMCOAST_MICROFINANCE_BANK = "090497"
CAPITALMETRIQ_SWIFT_MICROFINANCE_BANK = "090509"
BOI_MICROFINANCE_BANK = "090444"
SOLID_ALLIANZE_MICROFINANCE_BANK = "090506"
UMUNNACHI_MICROFINANCE_BANK = "090510"
CLOVER_LEAF_MICROFINANCE_BANK = "090511"
STANDARD_MICROFINANCE_BANK = "090182"
FIMS_MICROFINANCE_BANK = "090507"
BUBAYERO_MICORFINANCE_BANK = "090512"
PREMIUMTRUST_BANK = "000031"
MOMO_PAYMENT_SERVICES_BANK = "120003"
RIMA_GROWTH_PATHWAY_MICROFINANCE_BANK = "090515"
SMARTCASH_PAYMENT_SERVICE_BANK = "120004"
HOMEBASE_MORTGAGE_BANK = "070024"
BOROMU_MICROFINANCE_BANK = "090501"
NIGERIAN_PRISONS_MICROFINANCE_BANK = "090505"
BORNO_RENAISSANCE_MICROFINANCE_BANK = "090508"
SEAP_MICROFINANCE_BANK = "090513"
UMUCHINEMERE_PROCREDIT_MICROFINANCE_BANK = "090514"
NUMO_MICROFINANCE_BANK = "090516"
UHURU_MICROFINANCE_BANK = "090517"
AFEMAI_MICROFINANCE_BANK = "090518"
IBOM_FADAMA_MICROFINANCE_BANK = "090519"
IC_GLOBALMICROFINANCE_BANK = "090520"
FORESIGHT_MICROFINANCE_BANK = "090521"
CHASE_MICROFINANCE_BANK = "090523"
SOLIDROCK_MICROFINANCE_BANK = "090524"
TRIPLE_A_MICROFINANCE_BANK = "090525"
BANKLY_MICROFINANCE_BANK = "090529"
MONEY_MASTER_PSB = "120005"
RESIDENT_FINTECH_LIMITED = "110024"
KORAYPAY = "110022"
CAPRICORN_DIGITAL = "110023"
NETAPPS_TECHNOLOGY_LIMITED = "110025"
SPAY_BUSINESS = "110026"
CATLANDA_MICROFINANCE_BANK = "090498"
OJOKORO_MICROFINANCE_BANK = "090527"
NOMBA_FINANCIAL_SERVICES_LIMITED = "110028"
YELLO_DIGITAL_FINANCIAL_SERVICES = "110027"
FEWCHORE_FINANCE_COMPANY_LIMITED = "050002"
ARAMOKO_MICROFINANCE_BANK = "090307"
IKOYI_OSUN_MICROFINANCE_BANK = "090536"
CRESCENT_MICROFINANCE_BANK = "090526"
IBU_AJE_MICROFINANCE_BANK = "090488"
AKU_MICROFINANCE_BANK = "090531"
GWONG_MICROFINANCE_BANK = "090500"
BUD_INFRASTRUCTURE_LIMITED = "110021"
SUPREME_MICROFINANCE_BANK = "090564"
SAGEGREY_FINACE_LIMITED = "050003"
AKWA_SAVINGS_AND_LOANS_LIMITED = "070025"
ISUOFIA_MFB = "090353"
PENIEL_MICORFINANCE_BANK_LTD = "090379"
UDA_MICROFINANCE_BANK = "090403"
MGBIDI_MICROFINANCE_BANK = "090528"
CONFIDENCE_MICROFINANCE_BANK_LTD = "090530"
IBOLO_MICORFINANCE_BANK_LTD = "090532"
POLYIBADAN_MICROFINANCE_BANK = "090534"
RSU_MICROFINANCE_BANK = "090535"
LOBREM_MICROFINANCE_BANK = "090537"
BLUE_INVESTMENTS_MICROFINANCE_BANK = "090538"
HALO_MICROFINANCE_BANK = "090539"
AZTEC_MICROFINANCE_BANK = "090540"
EXCELLENT_MICROFINANCE_BANK = "090541"
OTUO_MICROFINANCE_BANK_LTD = "090542"
ASPIRE_MICROFINANCE_BANK_LTD = "090544"
ABULESORO_MICROFINANCE_BANK_LTD = "090545"
IJEBU_IFE_MICROFINANCE_BANK_LTD = "090546"
ROCKSHIELD_MICROFINANCE_BANK = "090547"
ALLY_MICROFINANCE_BANK = "090548"
GREEN_ENERGY_MICROFINANCE_BANK_LTD = "090550"
FAIRMONEY_MICROFINANCE_BANK_LTD = "090551"
EKIMOGUN_MICROFINANCE_BANK = "090552"
WOVEN_FINANCE = "110029"
E_NAIRA = "000033"
PAYSTACK_TITAN = "100039"
RANDALPHA_MFB = "090496"
KAYVEE_MICROFINANCE_BANK = "090554"
IYERU_OKIN__MICROFINANCE_BANK_LTD = "090337"
BROADVIEW_MICROFINANCE_BANK = "090568"
OAU_MICROFINANCE_BANK = "090345"
IYAMOYE_MICROFINANCE_BANK = "090570"
OCTOPUS_MICROFINANCE_BANK = "090576"
BRANCH_INTERNATIONAL_FINANCIAL_SERVICES = "050006"
NEWEDGE_FINANCE_LTD = "050004"
GBEDE_MICROFINANCE_BANK = "090579"
ORISUN_MICROFINANCE_BANK = "090588"
GABSYN_MICROFINANCE_BANK = "090591"
ENCO_FINANCE_COMPANY_LTD = "050012"
KANOPOLY_MICROFINANCE_BANK_LIMITED = "090592"
KENECHUKWU_MICROFINANCE_BANK = "090602"
IBA_MICROFINANCE_BANK = "090598"
OPTIMUS_BANK = "000036"
MERCURY_MICROFINANCE_BANK = "090589"
AKPO_MICROFINANCE_BANK = "090608"
TASUED_MICROFINANCE_BANK = "090593"
SIGNATURE_BANK = "000034"
TEKLA_FINANCE_LTD = "050007"
FAST_CREDIT = "050009"
FUNDQUEST_FINANCIAL_SERVICES_LTD = "050010"
DIGNITY_FINANCE = "050013"
FHA_MORTGAGE_BANK_LTD = "070026"
MKOBO_MICROFINANCE_BANK_LTD = "090455"
QUBE_MICROFINANCE_BANK_LTD = "090569"
EWT_MICROFINANCE_BANK = "090572"
SNOW_MFB = "090573"
GOLDMAN_MICROFINANCE_BANK_LTD = "090574"
FIRSTMIDAS_MICROFINANCE_BANK_LTD = "090575"
IWADE_MICROFINANCE_BANK_LTD = "090578"
OTECH_MICROFINANCE_BANK_LTD = "090580"
BANC_CORP_MICROFINANCE_BANK_LTD = "090581"
STATESIDE_MFB = "090583"
ISLAND_MICROFINANCE_BANK_LTD = "090584"
GOMBE_MICROFINANCE_BANK_LTD = "090586"
MICROBIZ_MFB = "090587"
WAYA_MICROFINANCE_BANK_LTD = "090590"
GREENACRES_MICROFINANCE_BANK_LTD = "090599"
AVE_MARIA_MICROFINANCE_BANK_LTD = "090600"
MACROD_MFB = "090603"
UMMAH_MICROFINANCE_BANK = "090609"
AMOYE_MICROFINANCE_BANK = "090610"
CREDITVILLE_MFB = "090611"
MEDEF_MFB = "090612"
BUILD_MICROFINANCE_BANK = "090613"
EGWAFIN_MICROFINANCE_BANK = "090556"
AKUCHUCKWU_MICROFINANCE_BANK = "90561"
BESTSTAR_MFB = "90615"
SHIELD_MICROFINANCE_BANK = "90559"
OROKAM_MICROFINANCE_BANK = "90567"
AAA_FINANCE = "050005"
KC_MICROFINANCE_BANK = "090549"
FLOURISH_MFB = "090614"
SHIELD_MICROFINANCE_BANK = "090559"
BESTSTAR_MICROFINANCE_BANK = "090615"
RAYYAN_MICROFINANCE_BANK = "090616"
TRINITY_FINANCIAL = "050014"
LOMA_MFB = "090620"
GIDAUNIYAR_ALHERI_MICROFINANCE_BANK = "090621"
MAB_ALLIANZ_MICROFINANCE_BANK = "090623"
KKU_MICROFINANCE_BANK = "090606"
LEADREMIT_LIMITED = "110044"
SIMPLE_FINANCE_LIMITED = "050008"
LUKEFIELD_FINANCE_COMPANY_LIMITED = "050015"
JAPAY_MICROFINANCE_BANK = "090629"
E_FINANCE = "050016"
GABASAWA_MICROFINANCE_BANK = "090582"
MADOBI_MICROFINANCE_BANK = "090605"
DAL_MICROFINANCE_BANK = "090596"
NSEHE_MICROFINANCE_BANK = "090628"
KOPO_KOPE_MICROFINANCE_BANK = "090617"
CARDINALSTONE_FINANCE = "050017"
OHHA_MICROFINANCE_BANK = "090626"
CASHBRIDGE_MICROFINANCE_BANK = "090634"
GLOBAL_INITIATIVE_MICROFINANCE_BANK = "090639"
BRIYTH_COVENANT_MICROFINANCE_BANK = "090636"
AWACASH_MICROFINANCE_BANK = "090633"
OYAN_MICROFINANCE_BANK = "090635"
MINJIBIR_MICROFINANCE_BANK = "090607"
MABINAS_MICROFINANCE_BANK = "090630"
BABURA_MICROFINANCE_BANK = "090625"
TANGALE_MICROFINANCE_BANK = "090638"
SOURCE_MICROFINANCE_BANK = "090641"
UNIFUND_MICROFINANCE_BANK_LIMITED = "090637"
AMUCHA_MICROFINANCE_BANK = "090645"
DIOBU_MICROFINANCE_BANK = "090643"
PROSPERITY_MICROFINANCE_BANK = "090642"
SUNTOP_MICROFINANCE_BANK = "090644"
XPRESS_WALLET = "100040"
CANAAN_MICROFINANCE_BANK = "090647"
LEADCITY_MICROFINANCE_BANK = "090650"
ODOAKPU_MICROFINANCE_BANK_LIMITED = "090654"
ABSU_MICROFINANCE_BANK = "090640"
OKENGWE_MICROFINANCE_BANK = "090646"
UMUCHUKWU_MICROFINANCE_BANK = "090652"
BUNKURE__MICROFINANCE_BANK = "090655"
NEXIM_BANK = "030001"
PYRAMID_MICROFINANCE_BANK = "090657"
VALE_FINANCE_LIMITED = "050020"
ALTERNATIVE_BANK_LIMITED = "000037"
AWESOME_MICROFINANCE_BANK = "090662"
GOOD_SHEPHERD_MICROFINANCE_BANK = "090664"
BUNDI_MICROFINANCE_BANK = "090661"
REVELATION_MICROFINANCE_BANK = "090666"
ENTITY_MICROFINANCE_BANK = "090656"
KADUPE_MICROFINANCE_BANK = "090669"
TREASURES_MICROFINANCE_BANK = "090663"
HABARIPAY = "110059"
IMSU_MICROFINANCE_BANK = "090670"
CRUST_MICROFINANCE_BANK = "090560"
DAVENPORT_MICROFINANCE_BANK = "090673"
EARNWELL_MICROFINANCE_BANK = "090674"
FINANCE_COMPANY_LIMITED = "110072"
CITYCODE_MORTGAGE_BANK = "070027"
NDDC_MICROFINANCE_BANK = "090679"
EXCEL_MICROFINANCE_BANK = "090678"
TRIVES_FINANCE_COMPANY_LTD = "050023"
PFI_FINANCE_COMPANY_LIMITED = "050021"
PATHFINDER_MICROFINANACE_BANK_LIMITED = "090680"
KAYI_MICROFINANCE_BANK = "090667"
BETHEL_MICROFINANCE_BANK = "090683"
BAM_MICROFINANCE_BANK = "090651"
IKOYI_ILE_MICROFINANCEBANK = "090681"
BUYPOWER_MICROFINANACE_BANK = "090682"
M_M_MFB = "090685"
MUTUAL_ALLIANCE_MORTGAGE_BANK = "070028"
NET_MFB = "090675"
GOSIFECHUKWU_MFB = "090687"
PROSPECTS_MFB = "090689"
OK_MICROFINANCE_BANK = "090567"
PRUDENT_MICROFINANCE_BANK = "090690"
URE_MICROFINANCE_BANK = "090619"
CSD_MFB = "090686"
NUGGETS_MFB = "090676"
WESLEY_MFB = "090699"
GARUN_MALLAM_MFB = "090691"
WRA_MFB = "090631"
BELLBANK_MICROFINANCE_BANK = "090672"
OLIVE_MFB = "090696"
BOKKOS_MFB = "090703"
GDL_FINANCE = "050026"
AKUCHUKWU_MFB = "090561"

BANKS_CHOICE = (
    (STERLING_BANK, "Sterling Bank"),
    (KEYSTONE_BANK, "Keystone Bank"),
    (FCMB, "FCMB"),
    (UNITED_BANK_FOR_AFRICA, "United Bank for Africa"),
    (JAIZ_BANK, "JAIZ Bank"),
    (FIDELITY_BANK, "Fidelity Bank"),
    (POLARIS_BANK, "POLARIS BANK"),
    (CITI_BANK, "Citi Bank"),
    (ECOBANK_BANK, "Ecobank Bank"),
    (UNITY_BANK, "Unity Bank"),
    (STANBICIBTC_BANK, "StanbicIBTC Bank"),
    (GTBANK_PLC, "GTBank Plc"),
    (ACCESS_BANK, "Access Bank"),
    (ZENITH_BANK, "Zenith Bank"),
    (FIRST_BANK_OF_NIGERIA, "First Bank of Nigeria"),
    (WEMA_BANK, "Wema Bank"),
    (UNION_BANK, "Union Bank"),
    (ENTERPRISE_BANK, "Enterprise Bank"),
    (HERITAGE, "Heritage"),
    (STANDARDCHARTERED, "StandardChartered"),
    (FORTIS_MICROFINANCE_BANK, "Fortis Microfinance Bank"),
    (COVENANT, "Covenant"),
    (ASOSAVINGS, "ASOSavings"),
    (JUBILEELIFE, "JubileeLife"),
    (TRUSTBOND_MORTGAGE_BANK_PLC, "Trustbond Mortgage Bank Plc"),
    (FETS, "FETS"),
    (PAGATECH, "Pagatech"),
    (PARKWAY_READYCASH, "Parkway-ReadyCash"),
    (OPAY_DIGITAL_SERVICES_LIMITED, "OPAY DIGITAL SERVICES LIMITED"),
    (ETRANZACT, "eTranzact"),
    (STANBIC_IBTC_EASE_WALLET, "Stanbic IBTC @ease Wallet"),
    (ECOMOBILE, "EcoMobile"),
    (GTMOBILE, "GTMobile"),
    (TEASYMOBILE, "TeasyMobile"),
    (MKUDI, "Mkudi"),
    (VTNETWORKS, "VTNetworks"),
    (ACCESS_MONEY, "Access Money"),
    (FIRSTMONIE_WALLET, "FirstMonie Wallet"),
    (KEGOW_CHAMSMOBILE, "KEGOW(CHAMSMOBILE)"),
    (FORTISMOBILE, "FortisMobile"),
    (HEDONMARK, "Hedonmark"),
    (ZENITHMOBILE, "ZenithMobile"),
    (FIDELITY_MOBILE, "Fidelity Mobile"),
    (MONEYBOX, "MoneyBox"),
    (EARTHOLEUM, "Eartholeum"),
    (FSDH, "FSDH"),
    (NIP_VIRTUAL_BANK, "NIP Virtual Bank"),
    (NPF_MICROFINANCE_BANK, "NPF MicroFinance Bank"),
    (SAFETRUST_MORTAGE, "Safetrust Mortage"),
    (CORONATION_MERCHANT_BANK, "Coronation Merchant Bank"),
    (TAGPAY_VISUAL_ICT, "Tagpay (Visual Ict)"),
    (GOMONEY, "GOMONEY"),
    (SUNTRUST_BANK, "Suntrust Bank"),
    (PAYATTITUDE_ONLINE, "PayAttitude Online"),
    (OMOLUABI, "Omoluabi"),
    (OMOLUABI, "Omoluabi"),
    (PAGE_MFBANK, "Page MFBank"),
    (FIRST_MORTGAGE_LIMITED, "First Mortgage Limited"),
    (NEW_PRUDENTIAL_BANK, "New Prudential Bank"),
    (IMPERIAL_HOMES_MORTGAGE_BANK, "Imperial Homes Mortgage Bank"),
    (CARBON_MFB, "Carbon MFB"),
    (INTELLIFIN, "Intellifin"),
    (PROVIDUS_BANK, "Providus Bank"),
    (VFD_MICROFINANCE_BANK, "VFD MICROFINANCE BANK"),
    (EKONDO_MICROFINANCE_BANK, "EKONDO MICROFINANCE BANK"),
    (BOCTRUST_MICROFINANCE_BANK_LIMITED, "Boctrust Microfinance Bank Limited"),
    (RAND_MERCHANT_BANK, "Rand Merchant Bank"),
    (TCF_MICROFINANCE_BANK_LIMITED, "TCF Microfinance Bank Limited"),
    (KONGAPAY, "KongaPay"),
    (AG_MORTGAGE_BANK_PLC, "AG Mortgage Bank Plc"),
    (IBILE_MICROFINANCE_BANK_LIMITED, "Ibile Microfinance Bank Limited"),
    (PARRALEX_MICROFINANCE_BANK, "Parralex Microfinance bank"),
    (FBNQUEST_MERCHANT_BANK, "FBNQuest Merchant bank"),
    (GATEWAY_MORTGAGE_BANK, "GATEWAY MORTGAGE BANK"),
    (ABBEY_MORTGAGE_BANK, "ABBEY MORTGAGE BANK"),
    (FINATRUST_MICROFINANCE_BANK, "FinaTrust Microfinance Bank"),
    (SEED_CAPITAL_MICROFINANCE_BANK, "Seed Capital Microfinance Bank"),
    (EMPIRE_MFB, "EMPIRE MFB"),
    (AMML_MFB, "AMML MFB"),
    (OHAFIA_MFB, "OHAFIA MFB"),
    (WETLAND_MFB, "WETLAND MFB"),
    (HASAL_MFB, "HASAL MFB"),
    (GOWANS_MFB, "GOWANS MFB"),
    (TRUSTBANC_J6_MICROFINANCE_BANK_LIMITED, "TrustBanc J6 Microfinance Bank Limited"),
    (XSLNCE_MICROFINANCE_BANK, "XSLNCE Microfinance Bank"),
    (REGENT_MFB, "REGENT MFB"),
    (FIDFUND_MFB, "FidFund MFB"),
    (BC_KASH_MFB, "BC Kash MFB"),
    (NDIORAH_MFB, "Ndiorah MFB"),
    (MONEYTRUST_MFB, "MONEYTRUST MFB"),
    (CONSUMER__MFB, "CONSUMER  MFB"),
    (INNOVECTIVES_KESH, "Innovectives Kesh"),
    (REFUGE_MORTGAGE_BANK, "Refuge Mortgage Bank"),
    (LBIC_MORTGAGE_BANK, "LBIC Mortgage Bank"),
    (PLATINUM_MORTGAGE_BANK, "PLATINUM MORTGAGE BANK"),
    (FIRST_GENERATION_MORTGAGE_BANK, "First Generation Mortgage Bank"),
    (BRENT_MORTGAGE_BANK, "Brent Mortgage Bank"),
    (INFINITY_TRUST__MORTGAGE_BANK, "Infinity trust  Mortgage Bank"),
    (ALLWORKERS_MFB, "ALLWORKERS MFB"),
    (RICHWAY_MFB, "RICHWAY MFB"),
    (AL_BARKAH_MFB, "AL-BARKAH MFB"),
    (ACCION_MFB, "ACCION MFB"),
    (PERSONAL_TRUST_MICROFINANCE_BANK, "Personal Trust Microfinance Bank"),
    (BAOBAB_MICROFINANCE_BANK, "BAOBAB MICROFINANCE BANK"),
    (PECAN_TRUST_MICROFINANCE_BANK, "Pecan Trust Microfinance Bank"),
    (ROYAL_EXCHANGE_MICROFINANCE_BANK, "Royal Exchange Microfinance Bank"),
    (VISA_MICROFINANCE_BANK, "Visa Microfinance Bank"),
    (SAGAMU_MICROFINANCE_BANK, "Sagamu Microfinance Bank"),
    (CHIKUM_MICROFINANCE_BANK, "Chikum Microfinance Bank"),
    (YES_MFB, "Yes MFB"),
    (APEKS_MICROFINANCE_BANK, "APEKS Microfinance Bank"),
    (MONEYFIELD_MICROFINANCE_BANK, "MONEYFIELD MICROFINANCE BANK"),
    (FULL_RANGE_MFB, "Full range MFB"),
    (TRIDENT_MICROFINANCE_BANK, "Trident Microfinance Bank"),
    (HACKMAN_MICROFINANCE_BANK, "Hackman Microfinance Bank"),
    (BOWEN_MFB, "Bowen MFB"),
    (IRL_MICROFINANCE_BANK, "IRL Microfinance Bank"),
    (VIRTUE_MFB, "Virtue MFB"),
    (MUTUAL_TRUST_MICROFINANCE_BANK, "Mutual Trust Microfinance Bank"),
    (NARGATA_MFB, "Nargata MFB"),
    (FFS_MICROFINANCE_BANK, "FFS Microfinance Bank"),
    (ADVANS_LAFAYETTE__MFB, "Advans LaFayette  MFB"),
    (E_BARCS_MFB, "e-BARCs MFB"),
    (INFINITY_MFB, "Infinity MFB"),
    (FUTO_MFB, "FUTO MFB"),
    (ADDOSSER_MFBB, "Addosser MFBB"),
    (FCMB_MOBILE, "FCMB MOBILE"),
    (FLUTTERWAVE_TECHNOLOGY_SOLUTIONS_LIMITED, "Flutterwave Technology solutions Limited"),
    (NOVA_MERCHANT_BANK, "NOVA MERCHANT BANK"),
    (FEDERAL_UNIVERSITY_DUTSE__MICROFINANCE_BANK, "FEDERAL UNIVERSITY DUTSE  MICROFINANCE BANK"),
    (KADPOLY_MICROFINANCE_BANK, "Kadpoly MICROFINANCE BANK"),
    (MAYFAIR_MFB, "Mayfair MFB"),
    (REPHIDIM_MICROFINANCE_BANK, "Rephidim MICROFINANCE BANK"),
    (IKENNE_MFB, "IKENNE MFB"),
    (TRUST_MFB, "Trust MFB"),
    (UNAAB_MFB, "UNAAB MFB"),
    (EVERGREEN_MICROFINANCE_BANK, "Evergreen MICROFINANCE BANK"),
    (BIPC_MICROFINANCE_BANK, "BIPC MICROFINANCE BANK"),
    (CASHCONNECT_MICROFINANCE_BANK, "Cashconnect MICROFINANCE BANK"),
    (NUTURE_MFB, "Nuture MFB"),
    (ILASAN_MICROFINANCE_BANK, "Ilasan MICROFINANCE BANK"),
    (AGOSASA_MICROFINANCE_BANK, "Agosasa MICROFINANCE BANK"),
    (LEGEND_MICROFINANCE_BANK, "Legend MICROFINANCE BANK"),
    (TF_MICROFINANCE_BANK, "TF MICROFINANCE BANK"),
    (COASTLINE_MICROFINANCE_BANK, "Coastline MICROFINANCE BANK"),
    (NEPTUNE_MFB, "Neptune MFB"),
    (MOLUSI_MFB, "MOLUSI MFB"),
    (MANNY_MFB, "Manny MFB"),
    (CORESTEP_MICROFINANCE_BANK, "Corestep MICROFINANCE BANK"),
    (SEEDVEST_MICROFINANCE_BANK, "SEEDVEST MICROFINANCE BANK"),
    (APPLE__MICROFINANCE_BANK, "APPLE  MICROFINANCE BANK"),
    (ISALEOYO_MICROFINANCE_BANK, "ISALEOYO MICROFINANCE BANK"),
    (NEW_GOLDEN_PASTURES_MICROFINANCE_BANK, "NEW GOLDEN PASTURES MICROFINANCE BANK"),
    (GTI__MICROFINANCE_BANK, "GTI  Microfinance Bank"),
    (EK_RELIABLE_MICROFINANCE_BANK, "EK-Reliable Microfinance Bank"),
    (DAVODANI__MICROFINANCE_BANK, "Davodani  Microfinance Bank"),
    (VENTURE_GARDEN_NIGERIA_LIMITED, "Venture Garden Nigeria Limited"),
    (ARCA_PAYMENTS_COMPANY_LIMITED, "ARCA PAYMENTS COMPANY LIMITED"),
    (MOZFIN_MICROFINANCE_BANK, "MOZFIN MICROFINANCE BANK"),
    (PAYMENT_SERVICE_BANK, "9 PAYMENT SERVICE BANK"),
    (INTERSWITCH_LIMITED, "Interswitch Limited"),
    (PAYSTACK_PAYMENTS, "Paystack Payments"),
    (TEAM_APT, "Team APT"),
    (ITEX_INTEGRATED_SERVICES_LIMITED, "Itex Integrated Services Limited"),
    (BORGU_MICROFINANCE_BANK, "BORGU MICROFINANCE BANK"),
    (HOPE_PAYMENT_SERVICE_BANK, "Hope Payment Service Bank"),
    (MONIEPOINT_MFB, "Moniepoint MFB"),
    (FEDERAL_POLYTECHNIC_NEKEDE_MICROFINANCE_BANK, "FEDERAL POLYTECHNIC NEKEDE MICROFINANCE BANK"),
    (MARITIME_MICROFINANCE_BANK, "MARITIME MICROFINANCE BANK"),
    (GMB_MICROFINANCE_BANK, "GMB Microfinance Bank"),
    (CHANELLE_MFB, "CHANELLE MFB"),
    (GIGINYA_MFB, "Giginya MFB"),
    (CHIBUEZE__MFB, "CHIBUEZE  MFB"),
    (PREEMINENT_MICROFINANCE_BANK, "Preeminent Microfinance Bank"),
    (COOP_MORTGAGE_BANK, "Coop Mortgage Bank"),
    (LANDGOLD_MICROFINANCE_BANK, "LANDGOLD MICROFINANCE BANK"),
    (UNN_MFB, "UNN MFB"),
    (ABUCOOP_MICROFINANCE_BANK, "ABUCOOP MICROFINANCE BANK"),
    (FIRMUS_MFB, "Firmus MFB"),
    (BENYSTA_MICROFINANCE_BANK, "BENYSTA MICROFINANCE BANK"),
    (PEACE_MICROFINANCE_BANK, "Peace Microfinance Bank"),
    (QR_PAYMENTS, "QR Payments"),
    (HIGHLAND_MICROFINANCE_BANK, "HIGHLAND MICROFINANCE BANK"),
    (CENTRAL_BANK_OF_NIGERIA, "Central Bank of Nigeria"),
    (MAUTECH_MICROFINANCE_BANK, "Mautech Microfinance Bank"),
    (TANGERINE_MONEY, "TANGERINE MONEY"),
    (ISHIE_MICROFINANCE_BANK, "Ishie Microfinance Bank"),
    (ILORA_MICROFINANCE_BANK, "Ilora Microfinance Bank"),
    (BLUEWHALES_MICROFINANCE_BANK, "Bluewhales Microfinance Bank"),
    (IZON_MICROFINANCE_BANK, "Izon Microfinance Bank"),
    (BANEX_MICROFINANCE_BANK, "Banex Microfinance Bank"),
    (LTSHEGO_MICROFINANCE_BANK, "Ltshego Microfinance Bank"),
    (SUNBEAM_MICROFINANCE_BANK, "Sunbeam Microfinance Bank"),
    (WINVIEW_BANK, "WinView Bank"),
    (CALABAR_MICROFINANCE_BANK, "Calabar Microfinance Bank"),
    (IMOWO_MICROFINANCE_BANK, "Imowo Microfinance Bank"),
    (FCMB_BETA, "FCMB BETA"),
    (BUSINESS_SUPPORTMICROFINANCE_BANK, "Business SupportMicrofinance Bank"),
    (SHEPHERD_TRUST_MICROFINANCE_BANK, "Shepherd Trust Microfinance Bank"),
    (DOJE_MICROFINANCE_BANK_LIMITED, "DOJE Microfinance Bank Limited"),
    (PETRA_MICROFINANCE_BANK_PLC, "PETRA MICROFINANCE BANK PLC"),
    (HAGGAI_MORTGAGE_BANK_LIMITED, "HAGGAI MORTGAGE BANK LIMITED"),
    (MALACHY_MICROFINANCE_BANK, "MALACHY MICROFINANCE BANK"),
    (MAINSTREET_MICROFINANCE_BANK, "MAINSTREET MICROFINANCE BANK"),
    (RUBIES_MICROFINANCE_BANK, "RUBIES MICROFINANCE BANK"),
    (FAST_MICROFINANCE_BANK, "FAST MICROFINANCE BANK"),
    (AMJU_UNIQUE_MICROFINANCE_BANK_, "AMJU UNIQUE MICROFINANCE BANK "),
    (ALPHA_KAPITAL_MFB_LTD, "Alpha Kapital Mfb Ltd"),
    (RENMONEY_MICROFINANCE_BANK, "RENMONEY MICROFINANCE BANK"),
    (BAINES_CREDIT_MICROFINANCE_BANK, "BAINES CREDIT MICROFINANCE BANK"),
    (NIGERIAN_NAVY_MICROFINANCE_BANK, "NIGERIAN NAVY MICROFINANCE BANK"),
    (KUDA_MICROFINANCE_BANK, "Kuda Microfinance Bank"),
    (NIRSAL_MICROFINANCE_BANK, "NIRSAL MICROFINANCE BANK"),
    (TRUSTFUND_MICROFINANCE_BANK, "Trustfund Microfinance Bank"),
    (RELIANCE_MFB, "Reliance MFB"),
    (NEW_DAWN_MICROFINANCE_BANK, "New Dawn Microfinance Bank"),
    (ESAN_MICROFINANCE_BANK, "Esan Microfinance Bank"),
    (OLABISI_ONABANJO_UNIVERSITY_MICROFINANCE_BANK, "OLABISI ONABANJO UNIVERSITY MICROFINANCE BANK"),
    (LINE_CARD_MANAGEMENT_LIMITED, "3line Card Management Limited"),
    (TITAN_TRUST_BANK, "TITAN TRUST BANK"),
    (FIRST_APPLE_LIMITED, "First Apple Limited"),
    (GLORY_MICROFINANCE_BANK, "Glory Microfinance Bank"),
    (GLOBUS_BANK, "Globus Bank"),
    (TAJ_BANK, "Taj Bank"),
    (AL_HAYAT_MICROFINANCE_BANK, "AL-HAYAT MICROFINANCE BANK"),
    (AB_MICROFINANCE_BANK, "AB Microfinance bank"),
    (ABOVE_ONLY_MICROFINANCE_BANK, "Above Only Microfinance bank"),
    (ABU_MICROFINANCE_BANK, "ABU Microfinance bank"),
    (ADEYEMI_COLLEGE_STAFF_MICROFINANCE_BANK, "Adeyemi College Staff Microfinance bank"),
    (ALEKUN_MICROFINANCE_BANK, "Alekun Microfinance bank"),
    (AUCHI_MICROFINANCE_BANK, "Auchi Microfinance bank"),
    (EMERALDS_MFB, "Emeralds MFB"),
    (GREENVILLE_MICROFINANCE_BANK, "Greenville Microfinance bank"),
    (GROOMING_MICROFINANCE_BANK, "Grooming Microfinance bank"),
    (IMO_MICROFINANCE_BANK, "Imo Microfinance bank"),
    (UNICAL_MFB, "Unical MFB"),
    (LAVENDER_MICROFINANCE_BANK, "Lavender Microfinance bank"),
    (LOVONUS_MICROFINANCE_BANK, "Lovonus Microfinance bank"),
    (KCMB_MFB, "KCMB MFB"),
    (PENNYWISE_MICROFINANCE_BANK, "Pennywise Microfinance bank"),
    (QUICKFUND_MICROFINANCE_BANK, "QuickFund Microfinance bank"),
    (STELLAS_MICROFINANCE_BANK, "Stellas Microfinance bank"),
    (UNIBEN_MICROFINANCE_BANK, "Uniben Microfinance bank"),
    (POLYUWANNA_MFB, "Polyuwanna MFB"),
    (ALERT_MFB, "Alert MFB"),
    (CEMCS_MFB, "CEMCS MFB"),
    (CREDIT_AFRIQUE_MFB, "Credit Afrique MFB"),
    (OKPOGA_MFB, "Okpoga MFB"),
    (STANFORD_MFB, "Stanford MFB"),
    (FIRST_MULTIPLE_MFB, "First Multiple MFB"),
    (MERIDIAN_MFB, "Meridian MFB"),
    (IKIRE_MFB, "Ikire MFB"),
    (MEGAPRAISE_MICROFINANCE_BANK, "Megapraise Microfinance Bank"),
    (MINT_FINEX_MICROFINANCE, "MINT-FINEX MICROFINANCE"),
    (ARISE_MFB, "Arise MFB"),
    (THRIVE_MFB, "THRIVE MFB"),
    (FIRST_OPTION_MFB, "First Option MFB"),
    (SAFE_HAVEN_MFB, "Safe Haven MFB"),
    (PILLAR_MFB, "Pillar MFB"),
    (FCT_MFB, "FCT MFB"),
    (HALA_MFB, "Hala MFB"),
    (EAGLE_FLIGHT_MFB, "Eagle Flight MFB"),
    (OMIYE_MFB, "Omiye MFB"),
    (ESO_E_MICROFINANCE_BANK, "Eso-E Microfinance Bank"),
    (DAYLIGHT_MICROFINANCE_BANK, "Daylight Microfinance Bank"),
    (GASHUA_MICROFINANCE_BANK, "Gashua Microfinance Bank"),
    (RAHAMA_MFB, "Rahama MFB"),
    (ASTRAPOLIS_MFB, "Astrapolis MFB"),
    (BOSAK_MFB, "Bosak MFB"),
    (PRESTIGE_MICROFINANCE_BANK, "Prestige Microfinance bank"),
    (GREENBANK_MFB, "GreenBank MFB"),
    (MUTUAL_BENEFIT_MFB, "MUTUAL BENEFIT MFB"),
    (ACCESS_YELLO_, "Access Yello "),
    (PALMPAY, "PALMPAY"),
    (AFEKHAFE_MICROFINANCE_BANK, "Afekhafe Microfinance Bank"),
    (SPARKLE, "Sparkle"),
    (PATRICKGOLD_MICROFINANCE_BANK, "PATRICKGOLD MICROFINANCE BANK"),
    (MAINLAND_MICROFINANCE_BANK, "MAINLAND MICROFINANCE BANK"),
    (BALOGUN_GAMBARI_MICROFINANCE_BANK, "BALOGUN GAMBARI MICROFINANCE BANK"),
    (KADICK_INTEGRATION_LIMITED, "Kadick Integration Limited"),
    (LAPO_MFB, "Lapo MFB"),
    (EYOWO_MFB, "EYOWO MFB"),
    (BRETHREN_MICROFINANCE_BANK, "BRETHREN MICROFINANCE BANK"),
    (OCHE_MICROFINANCE_BANK, "OCHE MICROFINANCE BANK"),
    (ACCELEREX_NETWORK, "Accelerex network"),
    (MAYFRESH_MORTGAGE_BANK, "MayFresh Mortgage Bank"),
    (XPRESS_PAYMENTS, "Xpress Payments"),
    (FEDERALPOLY_NASARAWAMFB, "FederalPoly NasarawaMFB"),
    (KONTAGORA_MFB, "Kontagora MFB"),
    (PURPLEMONEY_MFB, "Purplemoney MFB"),
    (EVANGEL_MFB, "Evangel MFB"),
    (SULSAP_MFB, "Sulsap MFB"),
    (BRIGHTWAY_MFB, "Brightway MFB"),
    (EDFIN_MFB, "Edfin MFB"),
    (U_AND_C_MICROFINANCE_BANK, "U AND C Microfinance Bank"),
    (BAYERO_MICROFINANCE_BANK, "Bayero MICROFINANCE BANK"),
    (AMAC_MICROFINANCE_BANK, "Amac Microfinance Bank"),
    (FINCA_MICROFINANCE_BANK, "Finca Microfinance Bank"),
    (BRIDGEWAY_MICROFINANCE_BANK, "Bridgeway Microfinance Bank"),
    (OSCOTECH_MFB, "Oscotech MFB"),
    (KREDI_MONEY_MICROFINANCE_BANK, "KREDI MONEY MICROFINANCE BANK"),
    (YOBE_MFB, "Yobe MFB"),
    (NWANNEGADI_MFB, "Nwannegadi MFB"),
    (GIREI_MFB, "Girei MFB"),
    (INTERLAND_MFB, "Interland MFB"),
    (HEADWAY_MFB, "Headway MFB"),
    (ASSETS_MATRIX_MFB, "Assets Matrix MFB"),
    (MIDLAND_MFB, "Midland MFB"),
    (MICROVIS_MICROFINANCE_BANK, "MICROVIS MICROFINANCE BANK"),
    (M36, "M36"),
    (ZENITH_EAZY_WALLET, "ZENITH EAZY WALLET"),
    (CYBERSPACE_LIMITED, "Cyberspace Limited"),
    (CELLULANT_PSSP, "Cellulant PSSP"),
    (LINKS_MICROFINANCE_BANK, "LINKS MICROFINANCE BANK"),
    (EBSU_MFB, "EBSU MFB"),
    (STB_MORTGAGE_BANK, "STB MORTGAGE BANK"),
    (MEMPHIS_MICROFINANCE_BANK, "MEMPHIS MICROFINANCE BANK"),
    (FUTMINNA_MICROFINANCE_BANK_, "Futminna Microfinance Bank "),
    (RIGO_MICROFINANCE_BANK, "RIGO MICROFINANCE BANK"),
    (UNILORIN_MICROFINANCE_BANK, "Unilorin Microfinance Bank"),
    (GREENWICH_MERCHANT_BANK, "GREENWICH MERCHANT BANK"),
    (LOTUS_BANK, "LOTUS BANK"),
    (FAME_MICROFINANCE_BANK, "FAME Microfinance Bank"),
    (INSIGHT_MICROFINANCE_BANK, "Insight Microfinance Bank"),
    (SPECTRUM_MICROFINANCE_BANK, "Spectrum Microfinance Bank"),
    (BORSTAL_MICROFINANCE_BANK, "Borstal Microfinance Bank"),
    (OSPOLY_MICROFINANCE_BANK, "Ospoly Microfinance Bank"),
    (CAPSTONE_MICROFINANCE_BANK, "CAPSTONE Microfinance Bank"),
    (PARALLEX_BANK, "PARALLEX BANK"),
    (KWASU_MICROFINANCE_BANK, "KWASU Microfinance Bank"),
    (NICE_MICROFINANCE_BANK, "NICE Microfinance Bank"),
    (IBETO_MICROFINANCE_BANK, "IBETO MICROFINANCE BANK"),
    (BALOGUN_FULANI_MICROFINANCE_BANK, "Balogun Fulani Microfinance Bank"),
    (BONGHE_MICROFINANCE_BANK, "Bonghe Microfinance Bank"),
    (GRANT_MICROFINANCE_BANK, "Grant Microfinance Bank"),
    (STOCKCORP__MICROFINANCE_BANK, "Stockcorp  Microfinance Bank"),
    (ILLORIN_MICROFINANCE_BANK, "Illorin Microfinance Bank"),
    (PARKWAY_MF_BANK, "Parkway MF Bank"),
    (CRUTECH__MICROFINANCE_BANK, "Crutech  Microfinance Bank"),
    (OAKLAND_MICROFINANCE_BANK, "Oakland Microfinance Bank"),
    (CHERISH_MICROFINANCE_BANK, "Cherish Microfinance Bank"),
    (GIWA_MICROFINANCE_BANK, "Giwa Microfinance Bank"),
    (RIMA_MICROFINANCE_BANK, "RIMA Microfinance Bank"),
    (SUPPORT_MF_BANK, "Support MF Bank"),
    (MOYOFADE_MF_BANK, "Moyofade MF Bank"),
    (REX_MICROFINANCE_BANK, "REX MICROFINANCE BANK"),
    (ATBU__MICROFINANCE_BANK, "ATBU  Microfinance Bank"),
    (UNILAG__MICROFINANCE_BANK, "Unilag  Microfinance Bank"),
    (UZONDU_MF_BANK, "Uzondu MF Bank"),
    (COALCAMP_MICROFINANCE_BANK, "CoalCamp Microfinance Bank"),
    (UNIUYO_MICROFINANCE_BANK, "UniUyo Microfinance Bank"),
    (CROSSRIVER__MICROFINANCE_BANK, "CrossRiver  Microfinance Bank"),
    (ORITA_BASORUN_MICROFINANCE_BANK, "ORITA-BASORUN Microfinance Bank"),
    (UNIIBADAN_MICROFINANCE_BANK, "UNIIBADAN Microfinance Bank"),
    (MONARCH_MICROFINANCE_BANK, "Monarch Microfinance Bank"),
    (REHOBOTH_MICROFINANCE_BANK, "Rehoboth Microfinance Bank"),
    (UNIMAID_MICROFINANCE_BANK, "Unimaid Microfinance Bank"),
    (MAINTRUST_MFB, "Maintrust MFB"),
    (GOOD_NEIGHBOURS_MICROFINANCE_BANK, "Good Neighbours Microfinance Bank"),
    (OLOFIN_OWENA_MICROFINANCE_BANK, "OLOFIN OWENA Microfinance Bank"),
    (CARETAKER_MICROFINANCE_BANK, "CARETAKER Microfinance Bank"),
    (ASSETS_MICROFINANCE_BANK, "ASSETS Microfinance Bank"),
    (SAFEGATE_MICROFINANCE_BANK, "Safegate Microfinance Bank"),
    (INTERSWITCH_FINANCIAL_INCLUSION_SERVICES_IFIS, "Interswitch Financial Inclusion Services (IFIS)"),
    (OLUCHUKWU_MICROFINANCE_BANK, "OLUCHUKWU Microfinance Bank"),
    (VAS2NETS_LIMITED, "Vas2nets Limited"),
    (CROWDFORCE, "CROWDFORCE"),
    (CHANGAN_RTS_MICROFINANCE_BANK, "Changan RTS Microfinance Bank"),
    (LIGHT_MICROFINANCE_BANK, "Light Microfinance Bank"),
    (AVUENEGBE_MFB, "Avuenegbe MFB"),
    (FEDETH_MFB, "FEDETH MFB"),
    (PRISCO_MFB, "PRISCO MFB"),
    (CINTRUST_MFB, "CINTRUST MFB"),
    (ADA_MFB, "ADA MFB"),
    (ANIOCHA_MFB, "Aniocha MFB"),
    (ALVANA_MICROFINANCE_BANK, "ALVANA MICROFINANCE BANK"),
    (GARKI_MFB, "GARKI MFB"),
    (FIRST_HERITAGE_MFB, "First Heritage MFB"),
    (FOTRESS_MICROFINANCE_BANK, "Fotress Microfinance Bank"),
    (ANCHORAGE_MFB, "Anchorage MFB"),
    (MICROSYSTEMS_INVESTMENT_AND_DEVELOPMENT_LIMITED, "Microsystems Investment and Development Limited"),
    (NIBSSUSSD_PAYMENTS, "NIBSSUSSD PAYMENTS"),
    (PRISTINE_DIVITIS_MFB, "Pristine Divitis MFB"),
    (GOODNEWS_MICROFINANCE_BANK, "GOODNEWS MICROFINANCE BANK"),
    (CHUKWUNENYE__MICROFINANCE_BANK, "Chukwunenye  Microfinance Bank"),
    (ORAUKWU_MICROFINANCE_BANK, "Oraukwu Microfinance Bank"),
    (KINGDOM_COLLEGE_MICROFINANCE_BANK, "KINGDOM COLLEGE MICROFINANCE BANK"),
    (JESSEFIELD_MICROFINANCE, "JESSEFIELD MICROFINANCE"),
    (DELTA_TRUST_MORTGAGE_BANK, "DELTA TRUST MORTGAGE BANK"),
    (IPERUA_MICROFINANCE_BANK, "IPERUÂ MICROFINANCE BANK"),
    (SHALOM_MICROFINACE_BANK, "SHALOM MICROFINACE BANK"),
    (NSUK_MICROFINANCE_BANK, "NSUK MICROFINANCE BANK"),
    (BOJIA_BOJIA_MICROFINANCE_BANK, "BOJIÂ BOJIÂ MICROFINANCE BANK"),
    (ZIKORA_MICROFINANCE_BANK, "ZIKORA MICROFINANCE BANK"),
    (TAJ_PINSPAY, "TAJ_PINSPAY"),
    (PROJECTS_MICROFINANCE_BANK, "PROJECTS MICROFINANCE BANK"),
    (NOWNOW_MODIFICATION, "NowNow (Modification)"),
    (CITIZEN_TRUST_A_MFB, "Citizen Trust Â MFB"),
    (FIRST_ROYAL_MICROFINANCE_BANK, "First Royal Microfinance Bank"),
    (PALMCOAST_MICROFINANCE_BANK, "PALMCOAST MICROFINANCE BANK"),
    (CAPITALMETRIQ_SWIFT_MICROFINANCE_BANK, "CAPITALMETRIQ SWIFT MICROFINANCE BANK"),
    (BOI_MICROFINANCE_BANK, "BOI Microfinance Bank"),
    (SOLID_ALLIANZE_MICROFINANCE_BANK, "SOLID ALLIANZE MICROFINANCE BANK"),
    (UMUNNACHI_MICROFINANCE_BANK, "Umunnachi Microfinance Bank"),
    (CLOVER_LEAF_MICROFINANCE_BANK, "Clover Leaf Microfinance Bank"),
    (STANDARD_MICROFINANCE_BANK, "Standard Microfinance Bank"),
    (FIMS_MICROFINANCE_BANK, "FIMS MICROFINANCE BANK"),
    (BUBAYERO_MICORFINANCE_BANK, "BUBAYERO MICORFINANCE BANK"),
    (PREMIUMTRUST_BANK, "PremiumTrust Bank"),
    (MOMO_PAYMENT_SERVICES_BANK, "Momo Payment services Bank"),
    (RIMA_GROWTH_PATHWAY_MICROFINANCE_BANK, "RIMA GROWTH PATHWAY MICROFINANCE BANK"),
    (SMARTCASH_PAYMENT_SERVICE_BANK, "SMARTCASH PAYMENT SERVICE BANK"),
    (HOMEBASE_MORTGAGE_BANK, "HomeBase Mortgage Bank"),
    (BOROMU_MICROFINANCE_BANK, "Boromu Microfinance Bank"),
    (NIGERIAN_PRISONS_MICROFINANCE_BANK, "NIGERIAN PRISONS MICROFINANCE BANK"),
    (BORNO_RENAISSANCE_MICROFINANCE_BANK, "Borno Renaissance Microfinance Bank"),
    (SEAP_MICROFINANCE_BANK, "SEAP Microfinance Bank"),
    (UMUCHINEMERE_PROCREDIT_MICROFINANCE_BANK, "Umuchinemere Procredit Microfinance Bank"),
    (NUMO_MICROFINANCE_BANK, "Numo Microfinance Bank"),
    (UHURU_MICROFINANCE_BANK, "Uhuru Microfinance Bank"),
    (AFEMAI_MICROFINANCE_BANK, "Afemai Microfinance Bank"),
    (IBOM_FADAMA_MICROFINANCE_BANK, "Ibom fadama Microfinance Bank"),
    (IC_GLOBALMICROFINANCE_BANK, "IC GLOBALMicrofinance Bank"),
    (FORESIGHT_MICROFINANCE_BANK, "Foresight Microfinance Bank"),
    (CHASE_MICROFINANCE_BANK, "Chase Microfinance Bank"),
    (SOLIDROCK_MICROFINANCE_BANK, "Solidrock Microfinance Bank"),
    (TRIPLE_A_MICROFINANCE_BANK, "Triple A Microfinance Bank"),
    (BANKLY_MICROFINANCE_BANK, "Bankly Microfinance Bank"),
    (MONEY_MASTER_PSB, "Money Master PSB"),
    (RESIDENT_FINTECH_LIMITED, "Resident Fintech Limited"),
    (KORAYPAY, "Koraypay"),
    (CAPRICORN_DIGITAL, "Capricorn Digital"),
    (NETAPPS_TECHNOLOGY_LIMITED, "NetApps Technology Limited"),
    (SPAY_BUSINESS, "SPAY Business"),
    (CATLANDA_MICROFINANCE_BANK, "CATLANDÂ MICROFINANCE BANK"),
    (OJOKORO_MICROFINANCE_BANK, "OJOKORO MICROFINANCE BANK"),
    (NOMBA_FINANCIAL_SERVICES_LIMITED, "Nomba financial services Limited"),
    (YELLO_DIGITAL_FINANCIAL_SERVICES, "Yello Digital financial services"),
    (FEWCHORE_FINANCE_COMPANY_LIMITED, "FEWCHORE FINANCE COMPANY LIMITED"),
    (ARAMOKO_MICROFINANCE_BANK, "ARAMOKO MICROFINANCE BANK"),
    (IKOYI_OSUN_MICROFINANCE_BANK, "IKOYI-OSUN MICROFINANCE BANK"),
    (CRESCENT_MICROFINANCE_BANK, "CRESCENT MICROFINANCE BANK"),
    (IBU_AJE_MICROFINANCE_BANK, "IBU-AJE MICROFINANCE BANK"),
    (AKU_MICROFINANCE_BANK, "AKU MICROFINANCE BANK"),
    (GWONG_MICROFINANCE_BANK, "GWONG MICROFINANCE BANK"),
    (BUD_INFRASTRUCTURE_LIMITED, "Bud infrastructure Limited"),
    (SUPREME_MICROFINANCE_BANK, "SUPREME MICROFINANCE BANK"),
    (SAGEGREY_FINACE_LIMITED, "SageGrey Finace Limited"),
    (AKWA_SAVINGS_AND_LOANS_LIMITED, "Akwa savings and Loans Limited"),
    (ISUOFIA_MFB, "Isuofia MFB"),
    (PENIEL_MICORFINANCE_BANK_LTD, "PENIEL MICORFINANCE BANK LTD"),
    (UDA_MICROFINANCE_BANK, "UDA Microfinance Bank"),
    (MGBIDI_MICROFINANCE_BANK, "Mgbidi Microfinance Bank"),
    (CONFIDENCE_MICROFINANCE_BANK_LTD, "CONFIDENCE MICROFINANCE BANK LTD"),
    (IBOLO_MICORFINANCE_BANK_LTD, "IBOLO MICORFINANCE BANK LTD"),
    (POLYIBADAN_MICROFINANCE_BANK, "Polyibadan Microfinance Bank"),
    (RSU_MICROFINANCE_BANK, "RSU MICROFINANCE BANK"),
    (LOBREM_MICROFINANCE_BANK, "Lobrem Microfinance Bank"),
    (BLUE_INVESTMENTS_MICROFINANCE_BANK, "Blue Investments Microfinance Bank"),
    (HALO_MICROFINANCE_BANK, "Halo Microfinance Bank"),
    (AZTEC_MICROFINANCE_BANK, "Aztec Microfinance Bank"),
    (EXCELLENT_MICROFINANCE_BANK, "EXCELLENT MICROFINANCE BANK"),
    (OTUO_MICROFINANCE_BANK_LTD, "OTUO MICROFINANCE BANK LTD"),
    (ASPIRE_MICROFINANCE_BANK_LTD, "ASPIRE MICROFINANCE BANK LTD"),
    (ABULESORO_MICROFINANCE_BANK_LTD, "ABULESORO MICROFINANCE BANK LTD"),
    (IJEBU_IFE_MICROFINANCE_BANK_LTD, "IJEBU-IFE MICROFINANCE BANK LTD"),
    (ROCKSHIELD_MICROFINANCE_BANK, "ROCKSHIELD MICROFINANCE BANK"),
    (ALLY_MICROFINANCE_BANK, "ALLY MICROFINANCE BANK"),
    (GREEN_ENERGY_MICROFINANCE_BANK_LTD, "GREEN ENERGY MICROFINANCE BANK LTD"),
    (FAIRMONEY_MICROFINANCE_BANK_LTD, "FAIRMONEY MICROFINANCE BANK LTD"),
    (EKIMOGUN_MICROFINANCE_BANK, "EKIMOGUN MICROFINANCE BANK"),
    (WOVEN_FINANCE, "Woven Finance"),
    (E_NAIRA, "e-Naira"),
    (PAYSTACK_TITAN, "Paystack-Titan"),
    (RANDALPHA_MFB, "RANDALPHA MFB"),
    (KAYVEE_MICROFINANCE_BANK, "Kayvee Microfinance Bank"),
    (IYERU_OKIN__MICROFINANCE_BANK_LTD, "IYERU OKIN  MICROFINANCE BANK LTD"),
    (BROADVIEW_MICROFINANCE_BANK, "BROADVIEW MICROFINANCE BANK"),
    (OAU_MICROFINANCE_BANK, "OAU MICROFINANCE BANK"),
    (IYAMOYE_MICROFINANCE_BANK, "IYAMOYE MICROFINANCE BANK"),
    (OCTOPUS_MICROFINANCE_BANK, "OCTOPUS MICROFINANCE BANK"),
    (BRANCH_INTERNATIONAL_FINANCIAL_SERVICES, "Branch International Financial Services"),
    (NEWEDGE_FINANCE_LTD, "NEWEDGE FINANCE LTD"),
    (GBEDE_MICROFINANCE_BANK, "GBEDE MICROFINANCE BANK"),
    (ORISUN_MICROFINANCE_BANK, "ORISUN MICROFINANCE BANK"),
    (GABSYN_MICROFINANCE_BANK, "GABSYN MICROFINANCE BANK"),
    (ENCO_FINANCE_COMPANY_LTD, "ENCO FINANCE COMPANY LTD"),
    (KANOPOLY_MICROFINANCE_BANK_LIMITED, "KANOPOLY MICROFINANCE BANK LIMITED"),
    (KENECHUKWU_MICROFINANCE_BANK, "KENECHUKWU MICROFINANCE BANK"),
    (IBA_MICROFINANCE_BANK, "IBA Microfinance Bank"),
    (OPTIMUS_BANK, "Optimus Bank"),
    (MERCURY_MICROFINANCE_BANK, "MERCURY MICROFINANCE BANK"),
    (AKPO_MICROFINANCE_BANK, "AKPO MICROFINANCE BANK"),
    (TASUED_MICROFINANCE_BANK, "TASUED MICROFINANCE BANK"),
    (SIGNATURE_BANK, "Signature Bank"),
    (TEKLA_FINANCE_LTD, "Tekla Finance Ltd"),
    (FAST_CREDIT, "FAST CREDIT"),
    (FUNDQUEST_FINANCIAL_SERVICES_LTD, "FUNDQUEST FINANCIAL SERVICES LTD"),
    (DIGNITY_FINANCE, "Dignity Finance"),
    (FHA_MORTGAGE_BANK_LTD, "FHA MORTGAGE BANK LTD"),
    (MKOBO_MICROFINANCE_BANK_LTD, "MKOBO MICROFINANCE BANK LTD"),
    (QUBE_MICROFINANCE_BANK_LTD, "QUBE MICROFINANCE BANK LTD"),
    (EWT_MICROFINANCE_BANK, "EWT Microfinance Bank"),
    (SNOW_MFB, "SNOW MFB"),
    (GOLDMAN_MICROFINANCE_BANK_LTD, "GOLDMAN MICROFINANCE BANK LTD"),
    (FIRSTMIDAS_MICROFINANCE_BANK_LTD, "FIRSTMIDAS MICROFINANCE BANK LTD"),
    (IWADE_MICROFINANCE_BANK_LTD, "IWADE MICROFINANCE BANK LTD"),
    (OTECH_MICROFINANCE_BANK_LTD, "OTECH MICROFINANCE BANK LTD"),
    (BANC_CORP_MICROFINANCE_BANK_LTD, "BANC CORP MICROFINANCE BANK LTD"),
    (STATESIDE_MFB, "STATESIDE MFB"),
    (ISLAND_MICROFINANCE_BANK_LTD, "ISLAND MICROFINANCE BANK LTD"),
    (GOMBE_MICROFINANCE_BANK_LTD, "GOMBE MICROFINANCE BANK LTD"),
    (MICROBIZ_MFB, "Microbiz MFB"),
    (WAYA_MICROFINANCE_BANK_LTD, "WAYA MICROFINANCE BANK LTD"),
    (GREENACRES_MICROFINANCE_BANK_LTD, "GREENACRES MICROFINANCE BANK LTD"),
    (AVE_MARIA_MICROFINANCE_BANK_LTD, "AVE MARIA MICROFINANCE BANK LTD"),
    (MACROD_MFB, "Macrod MFB"),
    (UMMAH_MICROFINANCE_BANK, "Ummah Microfinance Bank"),
    (AMOYE_MICROFINANCE_BANK, "AMOYE MICROFINANCE BANK"),
    (CREDITVILLE_MFB, "Creditville MFB"),
    (MEDEF_MFB, "Medef MFB"),
    (BUILD_MICROFINANCE_BANK, "BUILD MICROFINANCE BANK"),
    (EGWAFIN_MICROFINANCE_BANK, "EGWAFIN MICROFINANCE BANK"),
    (AKUCHUCKWU_MICROFINANCE_BANK, "AKUCHUCKWU MICROFINANCE BANK"),
    (BESTSTAR_MFB, "BESTSTAR MFB"),
    (SHIELD_MICROFINANCE_BANK, "SHIELD MICROFINANCE BANK"),
    (OROKAM_MICROFINANCE_BANK, "OROKAM MICROFINANCE BANK"),
    (AAA_FINANCE, "AAA FINANCE"),
    (KC_MICROFINANCE_BANK, "KC MICROFINANCE BANK"),
    (FLOURISH_MFB, "Flourish MFB"),
    (SHIELD_MICROFINANCE_BANK, "Shield Microfinance Bank"),
    (BESTSTAR_MICROFINANCE_BANK, "BESTSTAR MICROFINANCE BANK"),
    (RAYYAN_MICROFINANCE_BANK, "RAYYAN MICROFINANCE BANK"),
    (TRINITY_FINANCIAL, "TRINITY FINANCIAL"),
    (LOMA_MFB, "LOMA MFB"),
    (GIDAUNIYAR_ALHERI_MICROFINANCE_BANK, "GIDAUNIYAR ALHERI MICROFINANCE BANK"),
    (MAB_ALLIANZ_MICROFINANCE_BANK, "MAB ALLIANZ MICROFINANCE BANK"),
    (KKU_MICROFINANCE_BANK, "KKU Microfinance Bank"),
    (LEADREMIT_LIMITED, "LEADREMIT LIMITED"),
    (SIMPLE_FINANCE_LIMITED, "SIMPLE FINANCE LIMITED"),
    (LUKEFIELD_FINANCE_COMPANY_LIMITED, "LUKEFIELD FINANCE COMPANY LIMITED"),
    (JAPAY_MICROFINANCE_BANK, "9JAPAY MICROFINANCE BANK"),
    (E_FINANCE, "E-FINANCE"),
    (GABASAWA_MICROFINANCE_BANK, "GABASAWA MICROFINANCE BANK"),
    (MADOBI_MICROFINANCE_BANK, "MADOBI MICROFINANCE BANK"),
    (DAL_MICROFINANCE_BANK, "DAL MICROFINANCE BANK"),
    (NSEHE_MICROFINANCE_BANK, "NSEHE MICROFINANCE BANK"),
    (KOPO_KOPE_MICROFINANCE_BANK, "KOPO KOPE MICROFINANCE BANK"),
    (CARDINALSTONE_FINANCE, "Cardinalstone Finance"),
    (OHHA_MICROFINANCE_BANK, "OHHA MICROFINANCE BANK"),
    (CASHBRIDGE_MICROFINANCE_BANK, "CASHBRIDGE MICROFINANCE BANK"),
    (GLOBAL_INITIATIVE_MICROFINANCE_BANK, "GLOBAL INITIATIVE MICROFINANCE BANK"),
    (BRIYTH_COVENANT_MICROFINANCE_BANK, "BRIYTH-COVENANT MICROFINANCE BANK"),
    (AWACASH_MICROFINANCE_BANK, "AWACASH MICROFINANCE BANK"),
    (OYAN_MICROFINANCE_BANK, "OYAN MICROFINANCE BANK"),
    (MINJIBIR_MICROFINANCE_BANK, "MINJIBIR MICROFINANCE BANK"),
    (MABINAS_MICROFINANCE_BANK, "MABINAS MICROFINANCE BANK"),
    (BABURA_MICROFINANCE_BANK, "BABURA MICROFINANCE BANK"),
    (TANGALE_MICROFINANCE_BANK, "TANGALE MICROFINANCE BANK"),
    (SOURCE_MICROFINANCE_BANK, "SOURCE MICROFINANCE BANK"),
    (UNIFUND_MICROFINANCE_BANK_LIMITED, "UNIFUND MICROFINANCE BANK LIMITED"),
    (AMUCHA_MICROFINANCE_BANK, "AMUCHA MICROFINANCE BANK"),
    (DIOBU_MICROFINANCE_BANK, "DIOBU MICROFINANCE BANK"),
    (PROSPERITY_MICROFINANCE_BANK, "PROSPERITY MICROFINANCE BANK"),
    (SUNTOP_MICROFINANCE_BANK, "SUNTOP MICROFINANCE BANK"),
    (XPRESS_WALLET, "XPRESS WALLET"),
    (CANAAN_MICROFINANCE_BANK, "CANAAN MICROFINANCE BANK"),
    (LEADCITY_MICROFINANCE_BANK, "LEADCITY MICROFINANCE BANK"),
    (ODOAKPU_MICROFINANCE_BANK_LIMITED, "ODOAKPU MICROFINANCE BANK LIMITED"),
    (ABSU_MICROFINANCE_BANK, "ABSU MICROFINANCE BANK"),
    (OKENGWE_MICROFINANCE_BANK, "OKENGWE MICROFINANCE BANK"),
    (UMUCHUKWU_MICROFINANCE_BANK, "UMUCHUKWU MICROFINANCE BANK"),
    (BUNKURE__MICROFINANCE_BANK, "BUNKURE  MICROFINANCE BANK"),
    (NEXIM_BANK, "NEXIM BANK"),
    (PYRAMID_MICROFINANCE_BANK, "PYRAMID MICROFINANCE BANK"),
    (VALE_FINANCE_LIMITED, "VALE FINANCE LIMITED"),
    (ALTERNATIVE_BANK_LIMITED, "ALTERNATIVE BANK LIMITED"),
    (AWESOME_MICROFINANCE_BANK, "Awesome Microfinance Bank"),
    (GOOD_SHEPHERD_MICROFINANCE_BANK, "GOOD SHEPHERD MICROFINANCE BANK"),
    (BUNDI_MICROFINANCE_BANK, "BUNDI MICROFINANCE BANK"),
    (REVELATION_MICROFINANCE_BANK, "REVELATION MICROFINANCE BANK"),
    (ENTITY_MICROFINANCE_BANK, "ENTITY MICROFINANCE BANK"),
    (KADUPE_MICROFINANCE_BANK, "KADUPE MICROFINANCE BANK"),
    (TREASURES_MICROFINANCE_BANK, "TREASURES MICROFINANCE BANK"),
    (HABARIPAY, "Habaripay"),
    (IMSU_MICROFINANCE_BANK, "IMSU MICROFINANCE BANK"),
    (CRUST_MICROFINANCE_BANK, "CRUST MICROFINANCE BANK"),
    (DAVENPORT_MICROFINANCE_BANK, "DAVENPORT MICROFINANCE BANK"),
    (EARNWELL_MICROFINANCE_BANK, "Earnwell MICROFINANCE BANK"),
    (FINANCE_COMPANY_LIMITED, "78 FINANCE COMPANY LIMITED"),
    (CITYCODE_MORTGAGE_BANK, "CITYCODE MORTGAGE BANK"),
    (NDDC_MICROFINANCE_BANK, "NDDC MICROFINANCE BANK"),
    (EXCEL_MICROFINANCE_BANK, "EXCEL MICROFINANCE BANK"),
    (TRIVES_FINANCE_COMPANY_LTD, "TRIVES FINANCE COMPANY LTD"),
    (PFI_FINANCE_COMPANY_LIMITED, "PFI FINANCE COMPANY LIMITED"),
    (PATHFINDER_MICROFINANACE_BANK_LIMITED, "PATHFINDER MICROFINANACE BANK LIMITED"),
    (KAYI_MICROFINANCE_BANK, "KAYI MICROFINANCE BANK"),
    (BETHEL_MICROFINANCE_BANK, "BETHEL MICROFINANCE BANK"),
    (BAM_MICROFINANCE_BANK, "BAM MICROFINANCE BANK"),
    (IKOYI_ILE_MICROFINANCEBANK, "IKOYI ILE MICROFINANCEBANK"),
    (BUYPOWER_MICROFINANACE_BANK, "BUYPOWER MICROFINANACE BANK"),
    (M_M_MFB, "M&M MFB"),
    (MUTUAL_ALLIANCE_MORTGAGE_BANK, "MUTUAL ALLIANCE MORTGAGE BANK"),
    (NET_MFB, "NET MFB"),
    (GOSIFECHUKWU_MFB, "GOSIFECHUKWU MFB"),
    (PROSPECTS_MFB, "PROSPECTS MFB"),
    (OK_MICROFINANCE_BANK, "OK MICROFINANCE BANK"),
    (PRUDENT_MICROFINANCE_BANK, "PRUDENT MICROFINANCE BANK"),
    (URE_MICROFINANCE_BANK, "URE MICROFINANCE BANK"),
    (CSD_MFB, "CSD MFB"),
    (NUGGETS_MFB, "NUGGETS MFB"),
    (WESLEY_MFB, "WESLEY MFB"),
    (GARUN_MALLAM_MFB, "Garun Mallam MFB"),
    (WRA_MFB, "WRA MFB"),
    (BELLBANK_MICROFINANCE_BANK, "BELLBANK Microfinance Bank"),
    (OLIVE_MFB, "OLIVE MFB"),
    (BOKKOS_MFB, "BOKKOS MFB"),
    (GDL_FINANCE, "GDL FINANCE"),
    (AKUCHUKWU_MFB, "Akuchukwu MFB"),
)

BANKS_SORTED_CHOICES = sorted(BANKS_CHOICE, key=lambda x: x[1])


def get_x_auth_cred(client_no: str) -> bytes:
    return binascii.hexlify(client_no.encode()).zfill(16)


class AccountNumberUBATextInput(SmartTextWidget):
    def __init__(self, attrs=None):
        attrs = {
            "placeholder": _("Account Number"),
            **(attrs or {}),
        }
        super().__init__(attrs)


class AccountHolderNameUBATextInput(SmartTextWidget):
    def __init__(self, attrs=None):
        attrs = {
            "placeholder": _("Account Holder Name"),
            **(attrs or {}),
        }
        super().__init__(attrs)


class UBASelect(forms.Select):
    template_name = "django/forms/widgets/uba_select.html"

    def __init__(self, attrs=None):
        attrs = {
            "choices": BANKS_SORTED_CHOICES,
            **(attrs or {}),
        }
        super().__init__(attrs)


class UBANameEnquiryMultiWidget(MultiValueWidgetMixin, MultiWidget):
    template_name = "django/forms/widgets/uba.html"
    custom_render = True

    def __init__(self, attrs=None):
        widgets = (
            UBASelect(attrs),
            AccountNumberUBATextInput(attrs),
            AccountHolderNameUBATextInput(attrs),
        )
        super().__init__(widgets, attrs)

    def decompress(self, value):
        return list(value.values()) if value else [None, None, None, None, None]

    @property
    def media(self):
        extra = "" if settings.DEBUG else ".min"
        base = super().media
        return base + VersionMedia(
            js=[
                "admin/js/vendor/jquery/jquery%s.js" % extra,
                "admin/js/jquery.init.js",
                "jquery.compat%s.js" % extra,
                "https://cdnjs.cloudflare.com/ajax/libs/select2/4.1.0-rc.0/js/select2.min.js",
                "select2/ajax_select%s.js" % extra,
                "jquery.formset%s.js" % extra,
                "smart.formset%s.js" % extra,
            ],
            css={
                "all": [
                    "select2/select2.min.css",
                ]
            },
        )


class UBANameEnquiryField(ConfigurableSmartField, forms.MultiValueField):
    widget = UBANameEnquiryMultiWidget
    flex_field: "FlexFormField"

    def __init__(self, *args, **kwargs):
        fields = [
            forms.ChoiceField(choices=BANKS_SORTED_CHOICES),
            forms.CharField(),
            forms.CharField(),
        ]
        kwargs["template_name"] = "django/forms/uba.html"
        super().__init__(fields, *args, **kwargs)

    def compress(self, values):
        values.insert(0, dict(BANKS_CHOICE)[values[0]])
        values.append(self.flex_field.advanced.get("ignore_error", False))
        return dict(zip(["name", "uba_code", "number", "holder_name", "ignore_error"], values, strict=True))

    def validate(self, value):
        super().validate(value)

        try:
            _, bank_code, account_number, account_full_name, _ = value.values()
        except ValueError:
            raise ValidationError("ValueError: not enough values to unpack") from None

        generate = False

        for __ in range(1, 3):
            headers = {
                "AccessCode": self.get_token(generate),
                "Applcode": config.UBA_APPL_CODE,
                "Clientno": config.UBA_CLIENT_NO,
                "X-Auth-Cred": config.UBA_X_AUTH_CRED,
            }

            payload = {
                "receiverBankCode": bank_code,
                "receiverAccountNumber": account_number,
                "countryCode": "NG",
                "tranType": "nameenquiry",
            }
            try:
                response = requests.post(config.UBA_NAME_ENQUIRY_URL, headers=headers, json=payload, timeout=60)
            except (ReadTimeout, ConnectionError):
                continue

            if response.status_code == 200:
                jresponse = response.json()
                if jresponse.get("errorFlag") == FALSE and jresponse.get("statusCode") == "0":
                    if jresponse.get("customerName").lower() != account_full_name.lower():
                        valid_name = jresponse.get("customerName")
                        raise ValidationError(
                            f"Account holder name does not match: ({valid_name})",
                            code="name_mismatch",
                            params={"name": valid_name},
                        )
                    return

                error_message = jresponse.get("statusMsg") or jresponse.get("errDesc")
                error_code = jresponse.get("statusCode") or jresponse.get("errCode")
                if error_code == "4":
                    error_message = "Invalid account number"
                raise ValidationError(f"{error_message}: (error {error_code})")
            generate = True

        if not self.flex_field.advanced.get("ignore_error", False):
            raise ValidationError(
                "Cannot reach UBA server",
                code="uba_error",
            )

    def get_token(self, generate) -> str:
        token = cache.get("uba-token")
        if generate or not token:
            i = 0
            while i < 2:
                headers = {"Content-Type": "application/x-www-form-urlencoded"}
                auth = HTTPBasicAuth(config.UBA_CONSUMER_KEY, config.UBA_CONSUMER_SECRET)
                form_data = {"grant_type": "client_credentials"}
                try:
                    response = requests.post(
                        config.UBA_TOKEN_URL, data=form_data, headers=headers, auth=auth, timeout=60
                    )
                    jresponse = response.json()
                    token = f"{jresponse['token_type']} {jresponse['access_token']}"
                except MissingSchema:
                    raise ValidationError("Invalid UBA Api url") from None
                except (ReadTimeout, ConnectionError, KeyError):
                    i += 1
                    continue
                cache.set("uba-token", token)
                return token
        return token
